% Filtre Kalman pour la fusee qui decolle
% Introduction a la robotique mobile GLO-4001/GLO-7021
% (c) Philippe Giguere 2012
clearvars
clf
myFontSize = 14;

% Variables inconnues dcrivant le systme.
Sa      = 0.5; % Bruit sur l'acceleration
Slaser  = 10;  % Bruit sur le laser
xVrai = [0;0;0];

% Variables connues
P = zeros(3,3);
X = xVrai;

% Intervalle de temps
T = 0.1;  

% Matrices
F = [1 T 0; 0 1 0; 0 0 0];
G = [0.5*T^2; T; 1];
L = [3 0 0];
Cv = (Sa)^2;
Cw = Slaser^2;
  
a  = 5;   % Acceleration constante, en m/s^2

for iStep = 1:60
    % Simulation du systeme
    Accel = a + Sa*randn();
    xVrai(1) = xVrai(1) + xVrai(2)*T + 0.5*T^2*Accel;
    xVrai(2) = xVrai(2) + T*Accel;
    xVrai(3) = Accel;
    z  = 3*xVrai(1) + Slaser*randn; % mesure laser. Le 3 provient de l'equation de notre
                                    % fonction de capteur.
    
    % Et maintenant le filtre de Kalman
    U = a;
    
    
    % ====== Propagation =========
    X = F*X + G*U;
    P = F*P*F' + G*Cv*G';
    
    % ======== Mise-a-jour ========
    K = P*L'/(L*P*L'+Cw);
    X = X + K*(z-L*X);
    P = (eye(size(P,2))-K*L)*P;
    
    
    Ax(iStep) = xVrai(1);
    AEx(iStep) = X(1);
    ALaser(iStep) = z;
    ErreurEx(iStep) = X(1)-xVrai(1);
    ErreurLaser(iStep) = z/3-xVrai(1); % Le facteur 1/3 est pour inverser la fonction de capteur.
    ATime(iStep) = iStep*T;
end

subplot(3,1,[1 2]);
hl(3) = plot(ATime,AEx,'ko'); %,'MarkerFaceColor','g');
hold on;
hl(2) = plot(ATime,ALaser./3,'kx'); % Le facteur 1/3 est pour inverser la fonction de capteur.
hl(1) = plot(ATime,Ax,'k-');
xlabel('Temps (s)','FontSize',myFontSize);
ylabel('Altitude (m)','FontSize',myFontSize);
legend(hl,{'Altitude reelle' 'Altitude laser (fonction inverse)' 'Altitude Kalman'},'Location','NorthWest');
set(gca,'FontSize',myFontSize-2);

subplot(3,2,5)
hist(ErreurLaser);
title('Distribution des erreurs avec le laser');
xlabel('Erreur (m)');
c = axis();

subplot(3,2,6)
hist(ErreurEx);
title('Distribution des erreurs avec le filtre de Kalman');
xlim(c(1:2));
xlabel('Erreur (m)');

print -deps KalmanFusee.eps

